package de.brightbyte.abstraction;


/**
 * Provides access to a specific property for a class of objects
 */
public interface PropertyAccessor<T, V> {
	public static class Constant<V> implements PropertyAccessor<Object, V> {
		protected V value;
		
		public Constant(V value) {
			this.value = value;
		}
		
		public String toString() {
			return getName();
		}
		
		public String getName() {
			return String.valueOf(value);
		}

		@SuppressWarnings("unchecked")
		public Class<V> getType() {
			return value == null ? null : (Class<V>)value.getClass();
		}

		public V getValue(Object obj) {
			return value;
		}

		public boolean isMutable() {
			return false;
		}

		public void setValue(Object obj, V value) {
			throw new UnsupportedOperationException(); 
		}
	}
	
	public String getName();
	public V getValue(T obj);
	public void setValue(T obj, V value);
	public boolean isMutable();
	public Class<V> getType();
}
