package de.brightbyte.data;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

public abstract class AbstractMultiMap<K, V, C extends Collection<V>> implements MultiMap<K, V, C> {
	protected Map<K, C> map;
	
	public AbstractMultiMap() {
		map = new HashMap<K, C>();
	}
	

	public Map<K, C> componentMap() {
		return map;
	}
	
	/* (non-Javadoc)
	 * @see de.brightbyte.data.Dummy#putAll(de.brightbyte.data.MultiMap)
	 */
	public int putAll(MultiMap<K, V, ? extends Collection<V>> m) {
		int c = 0;
		
		for (Map.Entry<K, ? extends Collection<V>> e: m.entrySet()) {
			for (V v: e.getValue()) {
				if (put(e.getKey(), v)) c++;
			}
		}
		
		return c;
	}
	
	/* (non-Javadoc)
	 * @see de.brightbyte.data.Dummy#putAll(K,C)
	 */
	public int putAll(K key, C values) {
		int c = 0;
		for (V v: values) {
			if (put(key, v)) c++;
		}
		
		return c;
	}
	
	/* (non-Javadoc)
	 * @see de.brightbyte.data.Dummy#get(K)
	 */
	@SuppressWarnings("unchecked")
	public C get(Object key) {
		C list = map.get(key);
		return list == null ? emptyContainer() : list;
	}
	
	/* (non-Javadoc)
	 * @see de.brightbyte.data.Dummy#put(K, V)
	 */
	public boolean put(K key, V value) {
		C list = map.get(key);
		if (list==null) {
			list = newContainer();
			map.put(key, list);
		}
		
		return list.add(value);
	}

	protected abstract C newContainer();
	protected abstract C emptyContainer();
	
	@Override
	public String toString() {
		return map.toString();
	}

	/* (non-Javadoc)
	 * @see de.brightbyte.data.Dummy#clear()
	 */
	public void clear() {
		map.clear();
	}

	/* (non-Javadoc)
	 * @see de.brightbyte.data.Dummy#containsKey(K)
	 */
	public boolean containsKey(K key) {
		return map.containsKey(key);
	}

	/* (non-Javadoc)
	 * @see de.brightbyte.data.Dummy#contains(K,V)
	 */
	public boolean contains(K key, V value) {
		C c = get(key);
		
		if (c==null) return false;
		else return c.contains(value);
	}

	/* (non-Javadoc)
	 * @see de.brightbyte.data.Dummy#entrySet()
	 */
	public Set<Map.Entry<K, C>> entrySet() {
		return map.entrySet();
	}

	/* (non-Javadoc)
	 * @see de.brightbyte.data.Dummy#isEmpty()
	 */
	public boolean isEmpty() {
		return map.isEmpty();
	}

	/* (non-Javadoc)
	 * @see de.brightbyte.data.Dummy#keySet()
	 */
	public Set<K> keySet() {
		return map.keySet();
	}

	/* (non-Javadoc)
	 * @see de.brightbyte.data.Dummy#remove(K)
	 */
	public C remove(K key) {
		return map.remove(key);
	}

	/* (non-Javadoc)
	 * @see de.brightbyte.data.Dummy#size()
	 */
	public int size() {
		return map.size();
	}

	/* (non-Javadoc)
	 * @see de.brightbyte.data.Dummy#values()
	 */
	public Collection<C> values() {
		return (Collection<C>)map.values();
	}

	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + ((map == null) ? 0 : map.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof MultiMap))
			return false;
		
		final MultiMap other = (MultiMap) obj;
		
		if (other.size()!=size()) return false;
		
		for (Map.Entry<K, C> e: entrySet()) {
			C v = e.getValue();
			Object w = other.get(e.getKey());
			
			if (!v.equals(w)) return false;
		}
		
		return true;
	}

}
