package de.brightbyte.data;

import java.util.Collection;
import java.util.Comparator;

import de.brightbyte.abstraction.PropertyAccessor;

public class Optimum<T> implements Functor<T, Collection<T>> {

	protected Comparator<T> comparator;
	
	@SuppressWarnings("unchecked")
	public <V> Optimum(PropertyAccessor<T, V> accessor) {
		this(accessor, (Comparator<V>)NaturalComparator.instance); //XXX: unsafe, no way to make sure V is Comparable
	}	
	
	@SuppressWarnings("unchecked")
	public Optimum() {
		this((Comparator<T>)NaturalComparator.instance); //XXX: unsafe, no way to make sure T is Comparable
	}
	
	public <V> Optimum(PropertyAccessor<T, V> accessor, Comparator<V> comparator) {
		this(new PropertyComparator<T, V>(accessor, comparator));
	}	
	
	public Optimum(Comparator<T> comparator) {
		if (comparator == null) throw new NullPointerException(); 
		this.comparator = comparator;
	}

	public T apply(Collection<T> candidates) {
		T best = null;
		
		for (T obj: candidates) {
			if (best==null || comparator.compare(best, obj)<0) {
				best = obj;
			}
		}
		
		return best;
	}

}
