package de.brightbyte.data;

import java.util.Collection;
import java.util.Comparator;
import java.util.Map;

import de.brightbyte.abstraction.MapAbstractor;
import de.brightbyte.abstraction.MultiMapAbstractor;
import de.brightbyte.abstraction.PropertyAccessor;

public class PropertyComparator<T, V> implements Comparator<T> {

	protected PropertyAccessor<T, V> accessor;
	protected Comparator<V> comparator;
	
	@SuppressWarnings("unchecked")
	public static <V>PropertyComparator<Map<String, ?>, V> newMapEntryComparator(String property, Comparator<V> comparator) {
		PropertyAccessor<Map<String, ?>, V> accessor = MapAbstractor.accessor(property);
		return new PropertyComparator<Map<String, ?>, V> (accessor, comparator);
	}
	
	@SuppressWarnings("unchecked")
	public static <V, W extends Collection<V>>PropertyComparator<MultiMap<String, ?, Collection<?>>, V> newMultiMapEntryComparator(String property, Comparator<V> comparator, Functor<V, W> aggregator, Class<V> type) {
		PropertyAccessor<MultiMap<String, ?, Collection<?>>, V> accessor = MultiMapAbstractor.accessor(property, aggregator, type);
		return new PropertyComparator<MultiMap<String, ?, Collection<?>>, V> (accessor, comparator);
	}
	
	@SuppressWarnings("unchecked")
	public  PropertyComparator(PropertyAccessor<T, V> accessor) {
		this(accessor, (Comparator<V>)NaturalComparator.instance);  //XXX: unsafe, no way to make sure V is Comparable
	}
	
	public PropertyComparator(PropertyAccessor<T, V> accessor, Comparator<V> comparator) {
		if (accessor == null) throw new NullPointerException();
		if (comparator == null) throw new NullPointerException(); 
		
		this.accessor = accessor;
		this.comparator = comparator;
	}

	public int compare(T a, T b) {
		V x = accessor.getValue(a);
		V y = accessor.getValue(b);
		
		return comparator.compare(x, y);
	}

}
