package de.brightbyte.data.filter;

/**
 * Baseclass for filters that wrap an inner filter
 * 
 * @author daniel
 *
 * @param <T> type of objects the inner filter works on
 * @param <V> type of objects the outer filter works on
 */
public abstract class FilterWrapper<T, V> implements Filter<V> { 
	protected Filter<T> filter;
	
	public FilterWrapper(Filter<T> filter) {
		this.filter = filter;
	}
	
	public abstract boolean matches(V collection);

	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + ((filter == null) ? 0 : filter.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final FilterWrapper other = (FilterWrapper) obj;
		if (filter == null) {
			if (other.filter != null)
				return false;
		} else if (!filter.equals(other.filter))
			return false;
		return true;
	}
	
}
