package de.brightbyte.io;

import java.io.IOException;

import de.brightbyte.util.SystemUtils;

public abstract class AbstractOutput implements Output {

	private IOException error;

	private String linebreak = null;
	
	public void newline() throws IOException {
		if (linebreak==null) linebreak = SystemUtils.getPropertySafely("line.separator", "\r\n");
		write(linebreak);
	}

	public void println(Object obj) {
		try {
			write(String.valueOf(obj));
			newline();
		} catch (IOException e) {
			error = e;
		}
	}
	
	public IOException getError() {
		return error;
	}
	
	public boolean checkError() {
		return getError() != null;
	}

	public void writeln(String s) throws IOException {
		write(s);
		newline();
	}

	public Appendable append(CharSequence csq) throws IOException {
		write(csq.toString());
		return this;
	}

	public Appendable append(char c) throws IOException {
		write(String.valueOf(c));
		return this;
	}

	public Appendable append(CharSequence csq, int start, int end)
			throws IOException {
		write(csq.subSequence(start, end).toString());
		return this;
	}

}
