package de.brightbyte.io;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.brightbyte.data.cursor.DataCursor;
import de.brightbyte.text.Chunker;
import de.brightbyte.text.CsvLineChunker;
import de.brightbyte.util.ErrorHandler;
import de.brightbyte.util.PersistenceException;

public class ChunkingCursor implements DataCursor<List<String>> {
	
	protected DataCursor<String> source;
	protected Chunker chunker;
	
	protected ErrorHandler<ChunkingCursor, ParseException, PersistenceException> errorHandler;
	protected int line = 0;

	//XXX: Caveat: fields containing line breaks are not supported! use \n encoding
	
	public ChunkingCursor(InputStream in, String enc) throws UnsupportedEncodingException {
		this( new LineCursor(in, enc) );
	}

	public ChunkingCursor(Reader rd) {
		this(new LineCursor(rd));
	}
	
	public ChunkingCursor(BufferedReader reader) {
		this(new LineCursor(reader));
	}

	public ChunkingCursor(DataCursor<String> source) {
		this(source, CsvLineChunker.tsv);
	}
	
	public ChunkingCursor(DataCursor<String> source, Chunker chunker) {
		if (source==null) throw new NullPointerException();
		this.source = source;
		this.chunker = chunker;
	}

	public void close() {
			source.close();
	}

	public List<String> next() throws PersistenceException {
		while (true) {
			String s = source.next();
			line ++;
			
			while (s!=null && s.length()==0) {
				s = source.next(); //skip blank lines //XXX: optional? 
				line ++;
			}
			
			if (s==null) return null;
			List<String> row = split(s);
			if (row!=null) return row;
		}
	}
	
	public int getLine() {
		return line;
	}

	public void setParseErrorHandler(ErrorHandler<ChunkingCursor, ParseException, PersistenceException> errorHandler) {
		this.errorHandler = errorHandler;
	}
	
	protected List<String> split(String s) throws PersistenceException {
		try {
			Iterable<String> ss = chunker.chunk(s);
			if (ss instanceof List) return (List<String>)ss;  
			else if (ss instanceof Collection) return new ArrayList<String>((Collection<String>)ss);
			else {
				List<String> a = new ArrayList<String>();
				for (String x: ss) a.add(x);
				return a;
			}
		} catch (ParseException e) {
			if (errorHandler!=null) { 
				errorHandler.handleError(this, "failed to parse line "+line, e);
				return null;
			} else {
				throw new PersistenceException("failed to parse line "+line, e);
			}
		}
	}

	protected void finalize() {
		close();
	}
}
