package de.brightbyte.io;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.brightbyte.data.cursor.DataCursor;
import de.brightbyte.util.PersistenceException;

public class NamedColumnsCursor implements DataCursor<Map<String, String>> {
	
	protected DataCursor<List<String>> source;
	private String[] fields;

	public NamedColumnsCursor(InputStream in, String enc) throws UnsupportedEncodingException {
		this( new ChunkingCursor(in, enc) );
	}

	public NamedColumnsCursor(Reader rd) {
		this(new ChunkingCursor(rd));
	}
	
	public NamedColumnsCursor(BufferedReader reader) {
		this(new ChunkingCursor(reader));
	}

	public NamedColumnsCursor(LineCursor lines) {
		this(new ChunkingCursor(lines));
	}

	public NamedColumnsCursor(DataCursor<List<String>> source) {
		if (source==null) throw new NullPointerException();
		this.source = source;
	}
	
	public void setFields(String[] fields) {
		if (fields==null) throw new NullPointerException();
		this.fields = fields;
	}

	public void readFields() throws PersistenceException {
		List<String> s = source.next();
		if (s!=null) {
			fields = (String[]) s.toArray(new String[s.size()]);
		}
	}

	public String[] getFields() {
		return fields;
	}
	
	public void close() {
			source.close();
	}

	public Map<String, String> next() throws PersistenceException {
		List<String> s = source.next();
		if (s==null) return null;
		return record(s);
	}

	protected Map<String, String> record(List<String> s) {
		if (fields==null) throw new IllegalStateException("call setFields() or readFields() first!");
		
		HashMap<String, String> m = new HashMap<String, String>(fields.length);
		
		for (int i=0; i<fields.length; i++) {
			m.put(fields[i], s.get(i)); 
		}

		return m;
	}

	protected void finalize() {
		close();
	}
}
