package de.brightbyte.io;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

public class OutputPrintWriter extends PrintWriter {

	protected Output output;
	private IOException error;

	public OutputPrintWriter(OutputStream out, Output output) {
		super(ConsoleIO.writer); //dummy
		this.output = output;
	}
	
	public IOException getError() {
		return error;
	}
	
	protected void setError(IOException error) {
		if (error==null) throw new NullPointerException();
		this.error = error;
		setError();
	}

	public PrintWriter append(char c)  {
		try {
			output.append(c);
		} catch (IOException e) {
			setError(e);
		}
		return this;
	}

	public PrintWriter append(CharSequence csq, int start, int end) {
		try {
			output.append(csq, start, end);
		} catch (IOException e) {
			setError(e);
		}
		return this;
	}

	public PrintWriter append(CharSequence csq) {
		try {
			output.append(csq);
		} catch (IOException e) {
			setError(e);
		}
		return this;
	}

	public void close() {
		try {
			output.close();
		} catch (IOException e) {
			setError(e);
		}
	}

	public void flush() {
		try {
			output.flush();
		} catch (IOException e) {
			setError(e);
		}
	}

	public void newline() throws IOException {
		output.newline();
	}

	public void println(Object obj) {
		output.println(obj);
	}

	public void write(String s) {
		try {
			output.write(s);
		} catch (IOException e) {
			setError(e);
		}
	}

	public void writeln(String s) throws IOException {
		output.writeln(s);
	}
	

	@Override
	public void write(char[] buf, int off, int len) {
			write(new String(buf, off, len));
	}

	@Override
	public void write(int b) {
			write(new String(new char[] { (char) b}));
	}
	
}
