package de.brightbyte.io;

import java.util.zip.DataFormatException;
import java.util.zip.Deflater;
import java.util.zip.Inflater;

import de.brightbyte.data.ByteString;

public class ZLIBDataCodec  implements BlockCodec<byte[]> {
	protected Deflater deflater = new Deflater();
	protected Inflater inflater = new Inflater();

	private byte[] ibuffer = null;
	private byte[] obuffer = null;
	
	public ZLIBDataCodec() {
	}
	
	protected byte[] assertInputBufferCapacity(int req, int min) {
		if (ibuffer==null || req>ibuffer.length) ibuffer = new byte[Math.max(min, req)];
		return ibuffer;
	}
	
	protected byte[] assertOutputBufferCapacity(int req, int min) {
		if (obuffer==null || req>obuffer.length) obuffer = new byte[Math.max(min, req)];
		return obuffer;
	}
	
	public int encode(byte[] obj, byte[] buffer, int offset) {
		if (obj.length==0) return 0;
		
		 deflater.reset();
		 deflater.setInput(obj);
		 deflater.finish();
		 int c = deflater.deflate(buffer, offset, buffer.length);
		 if (c==0) throw new RuntimeException("deflater does not have enough data");
		 return c;
	}

	public ByteString encode(byte[] obj) {
		if (obj.length==0) return ByteString.EMPTY;
		
		byte[] buffer = assertOutputBufferCapacity(obj.length, 128);
		
		 deflater.setInput(obj);
		 deflater.finish();
		 int c = deflater.deflate(buffer);
		 if (c==0) throw new RuntimeException("deflater does not have enough data");

		 return new ByteString(buffer, 0, c);
	}

	public void setLevel(int level) {
		deflater.setLevel(level);
	}

	public void setStrategy(int strategy) {
		deflater.setStrategy(strategy);
	}


	public byte[] decode(byte[] data, int offset, int length) throws DataFormatException {
		if (length==0) return new byte[0];
		
		inflater.reset();
		inflater.setInput(data, offset, length);
		
		byte[] buffer = assertOutputBufferCapacity(data.length*10, 512);
		int len = inflater.inflate(buffer);
		
		if (len==0) {
			if (inflater.needsDictionary()) throw new RuntimeException("inflater needs dictionary");
			if (inflater.needsInput()) throw new RuntimeException("inflater needs more input");
		}
		
		byte[] result = new byte[len];
		System.arraycopy(buffer, 0, result, 0, len);
		
		return result;
	}

	public byte[] decode(ByteString b) throws DataFormatException {
		if (b.length()==0) return new byte[0];

		byte[] buffer = assertOutputBufferCapacity(b.length(), 128);
		b.getBytes(0, b.length(), buffer, 0);
		
		return decode(ibuffer, 0, b.length());
	}
	
}
