package de.brightbyte.io;

import java.io.UnsupportedEncodingException;
import java.util.zip.DataFormatException;

import de.brightbyte.data.ByteString;

public class ZLIBStringCodec implements BlockCodec<String> {
	protected ZLIBDataCodec codec = new ZLIBDataCodec();
	protected String charset;
	
	public ZLIBStringCodec(String charset) throws UnsupportedEncodingException {
		if (charset==null) throw new NullPointerException("charset must not be null");
		"".getBytes(charset); //test charset
		
		this.charset = charset;
	}
	
	protected String bytesToString(byte[] b) {
		try {
			return new String(b, charset);
		} catch (UnsupportedEncodingException e) {
			throw new Error("charset no longer supported: "+e); //charset was tested in the constructor. 
		}
	}

	protected byte[] stringToBytes(String s) {
		try {
			return s.getBytes(charset);
		} catch (UnsupportedEncodingException e) {
			throw new Error("charset no longer supported: "+e); //charset was tested in the constructor. 
		}
	}

	public String decode(byte[] data, int offset, int length)
			throws DataFormatException {
		
		byte[] b = codec.decode(data, offset, length);
		return bytesToString(b);
	}

	public String decode(ByteString bs) throws DataFormatException {
		byte[] b = codec.decode(bs);
		return bytesToString(b);
	}

	public ByteString encode(String s) {
		byte[] b = stringToBytes(s);
		return codec.encode(b);
	}

	public int encode(String s, byte[] buffer, int offset) {
		byte[] b = stringToBytes(s);
		return codec.encode(b, buffer, offset);
	}

	public void setLevel(int level) {
		codec.setLevel(level);
	}

	public void setStrategy(int strategy) {
		codec.setStrategy(strategy);
	}

}
