package de.brightbyte.io;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.zip.DataFormatException;
import java.util.zip.Deflater;
import java.util.zip.Inflater;

import de.brightbyte.util.StringUtils;

public class ZLIBStringCodecBenchmark {
	public static int compressLines(File s, File d, String enc) throws IOException {
		FileInputStream in = new FileInputStream(s);
		FileOutputStream out = new FileOutputStream(d);
		
		int c = compressLines(in, out, enc);
		
		out.close();
		in.close();
		
		return c;
	}
	
	public static int uncompressLines(File s, File d, String enc) throws IOException, DataFormatException {
		FileInputStream in = new FileInputStream(s);
		FileOutputStream out = new FileOutputStream(d);
		
		int c = uncompressLines(in, out, enc);
		
		out.close();
		in.close();
		
		return c;
	}
	
		public static int compressLines(InputStream in, OutputStream out, String enc) throws IOException {
			ZLIBStringCodec codec = new ZLIBStringCodec (enc); 
			
			BufferedReader rd = new BufferedReader(new InputStreamReader(in));
			DataOutputStream dout = new DataOutputStream(new BufferedOutputStream(out));
			
			byte[] buffer = new byte[256];
			String s;
			int count = 0;
			while ((s=rd.readLine())!=null) {
				if (buffer.length<s.length()*4) buffer = new byte[s.length()*4];
				int c = codec.encode(s, buffer, 0);
				dout.writeInt(c);
				dout.write(buffer, 0, c);
				count++;
			}
			
			dout.writeInt(-1);
			dout.close();
			rd.close();
			
			return count;
		}

		public static int uncompressLines(InputStream in, OutputStream out, String enc) throws IOException, DataFormatException {
			ZLIBStringCodec codec = new ZLIBStringCodec (enc);
			DataInputStream din = new DataInputStream(new BufferedInputStream(in));
			Writer wr = new OutputStreamWriter(new BufferedOutputStream(out));
			
			int count = 0;
			byte[] buffer = new byte[256];
			while (true) {
				if (din.available()<0) break;
				int c = din.readInt();
				if (c<0) break;

				if (buffer.length<c) buffer = new byte[c];
				c = din.read(buffer, 0, c);
				
				String s  = codec.decode(buffer, 0, c);
				
				wr.write(s);
				wr.write('\n');
				count++;
			}
			
			wr.close();
			din.close();
			
			return count;
		}
		
		public static void main(String[] args) throws IOException, DataFormatException {
			 try {
			     // Encode a String into bytes
			     //String inputString = "blahblahblah??";
				 String inputString = "x";
			     System.out.println("INPUT: "+inputString);

			     byte[] input = inputString.getBytes("UTF-8");

			     // Compress the bytes
			     byte[] output = new byte[100];
			     Deflater compresser = new Deflater();
			     compresser.setInput(input);
			     compresser.finish();
			     int compressedDataLength = compresser.deflate(output);

			     System.out.println("COMPRESSED["+compressedDataLength+"]: "+StringUtils.hex(output, 0, compressedDataLength));
			     
			     // Decompress the bytes
			     Inflater decompresser = new Inflater();
			     decompresser.setInput(output, 0, compressedDataLength);
			     byte[] result = new byte[100];
			     int resultLength = decompresser.inflate(result);
			     decompresser.end();

			     // Decode the bytes into a String
			     String outputString = new String(result, 0, resultLength, "UTF-8");
			     System.out.println("OUTPUT: "+outputString);
			 } catch(java.io.UnsupportedEncodingException ex) {
			     // handle
			 } catch (java.util.zip.DataFormatException ex) {
			     // handle
			 }
			 
			// System.exit(0);
			
			///////////////////////////////////
			String path = args[0];
			File f = new File(path);
			File z = new File(path+".compressed");
			File r = new File(path+".uncompressed");
			
			System.out.println("original: "+f.length());
			
			long t= System.currentTimeMillis();
			int c = compressLines(f, z, "UTF-8");
			System.out.println("compress("+c+"): "+(System.currentTimeMillis()-t)+"ms, "+z.length());

			t= System.currentTimeMillis();
			c = uncompressLines(z, r, "UTF-8");
			System.out.println("uncompress("+c+"): "+(System.currentTimeMillis()-t)+"ms, "+r.length());
		}
}
