package de.brightbyte.text;

import java.text.ParseException;
import java.text.ParsePosition;
import java.util.ArrayList;
import static de.brightbyte.text.DefaultStringLiteralParser.*;

public class CsvLineChunker implements Chunker {

	public static final int escapeFlags = ALLOW_NAMED_ESCAPE | ALLOW_HEX_ESCAPE | ALLOW_UNICODE_ESCAPE;
	public static final int literalParserFlags = escapeFlags | ALLOW_REPEATED_QUOTE | ALLOW_LINE_BREAKS | ALLOW_ESCAPED_LINE_BREAKS;
	public static final int fieldParserFlags = escapeFlags | ALLOW_ESCAPED_LINE_BREAKS | IMPLICIT_INITIAL_QUOTE | IMPLICIT_FINAL_QUOTE;
	
	public static final CsvLineChunker csv = new CsvLineChunker(',', false);
	public static final CsvLineChunker tsv = new CsvLineChunker('\t', true);
	
	protected StringLiteralParser fieldParser;
	protected StringLiteralParser literalParser;
	
	public CsvLineChunker() {
		this(',', false);
	}
	
	public CsvLineChunker(char separator, boolean esc) {
		this(separator, "\"", esc);		
	}
	
	public CsvLineChunker(char separator, String quotes, boolean esc) {
		this(new DefaultStringLiteralParser(quotes, esc ? '\\' : '\0', literalParserFlags), 
				new DefaultStringLiteralParser(""+separator, esc ? '\\' : '\0', fieldParserFlags));		
	}
	
	public CsvLineChunker(StringLiteralParser literalParser, StringLiteralParser fieldParser) {
		this.literalParser = literalParser;
		this.fieldParser = fieldParser;
	}

	public ArrayList<String> chunk(String s) throws ParseException {
		ParsePosition pos = new ParsePosition(0);

		ArrayList<String> chunks = new ArrayList<String>();
		
		while (pos.getIndex()<s.length()) {
			char ch = s.charAt(pos.getIndex());
			
			String chunk;
			if (literalParser.getQuoteChars().indexOf(ch)>=0) {
				chunk = literalParser.parseStringLiteral(s, pos);
				chunks.add(chunk);
				
				if (pos.getIndex()<s.length()) { //check for separator
					ch = s.charAt(pos.getIndex());
					if (fieldParser.getQuoteChars().indexOf(ch)<0)
						throw new ParseException("expected separator after closing quote!", pos.getIndex());
					pos.setIndex(pos.getIndex()+1);
				}
			}
			else {
				chunk = fieldParser.parseStringLiteral(s, pos);
				chunks.add(chunk);
				
				if (pos.getIndex()==s.length()) { //last column, check for final separator
					ch = s.charAt(pos.getIndex()-1);
					if (fieldParser.getQuoteChars().indexOf(ch)>=0)
						chunks.add("");
				}
			}
			
		}
		
		return chunks;
	}

}
