package de.brightbyte.text;

import java.text.ParseException;
import java.util.Arrays;

import junit.framework.TestCase;

public class CsvLineChunkerTest extends TestCase {

	public void testChunk_csv() throws ParseException {
		assertEquals(
				Arrays.asList(new String[] {}), 
				CsvLineChunker.csv.chunk("") );

		assertEquals(
				Arrays.asList(new String[] {"", "", ""}), 
				CsvLineChunker.csv.chunk(",,") );

		assertEquals(
				Arrays.asList(new String[] {"a", "", "c", ""}), 
				CsvLineChunker.csv.chunk("a,,c,") );

		assertEquals(
				Arrays.asList(new String[] {"a", "", "c"}), 
				CsvLineChunker.csv.chunk("\"a\",\"\",\"c\"") );

		assertEquals(
				Arrays.asList(new String[] {"a,x", "", "\"c\"x"}), 
				CsvLineChunker.csv.chunk("\"a,x\",,\"\"\"c\"\"x\"") );

		assertEquals(
				Arrays.asList(new String[] {"a\nx", "", "c\tx"}), 
				CsvLineChunker.csv.chunk("\"a\nx\",,\"c\tx\"") );

		assertEquals(
				Arrays.asList(new String[] {"a\\nx", "", "c\\tx"}), 
				CsvLineChunker.csv.chunk("\"a\\nx\",,c\\tx") );
	}

	public void testChunk_tsv() throws ParseException {
		assertEquals(
				Arrays.asList(new String[] {}), 
				CsvLineChunker.tsv.chunk("") );

		assertEquals(
				Arrays.asList(new String[] {"", "", ""}), 
				CsvLineChunker.tsv.chunk("\t\t") );

		assertEquals(
				Arrays.asList(new String[] {"a", "", "c", ""}), 
				CsvLineChunker.tsv.chunk("a\t\tc\t") );

		assertEquals(
				Arrays.asList(new String[] {"a", "", "c"}), 
				CsvLineChunker.tsv.chunk("\"a\"\t\"\"\t\"c\"") );

		assertEquals(
				Arrays.asList(new String[] {"a\tx", "", "\"c\"x"}), 
				CsvLineChunker.tsv.chunk("\"a\tx\"\t\t\"\"\"c\"\"x\"") );

		assertEquals(
				Arrays.asList(new String[] {"a\nx", "", "c\tx"}), 
				CsvLineChunker.tsv.chunk("\"a\nx\"\t\t\"c\tx\"") );

		assertEquals(
				Arrays.asList(new String[] {"a\nx", "", "c\tx"}), 
				CsvLineChunker.tsv.chunk("\"a\\nx\"\t\tc\\tx") );
	}

}
