package de.brightbyte.util;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public class CollectionUtils {

	public static <K,V> Map<K, V> deepMerge(Map<K, V> a, Map<K, V> b) {
		for (Map.Entry<K, V> e: b.entrySet()) {
			K k = e.getKey();
			V vb = e.getValue();
			
			if (vb==null) continue;
			
			V va = a.get(k);
			
			if (va==null) {
				a.put(k, vb);
			}
			else if (va instanceof Map) {
				if (vb instanceof Map) {
					deepMerge((Map)va, (Map)vb);
				}
				else {
					throw new IllegalArgumentException("can't merge instance of "+vb.getClass()+" into map (key was: "+k+")");
				}
			}
			else if (va instanceof Collection) {
				if (vb instanceof Collection) {
					((Collection)va).addAll((Collection)vb);
				}
				else {
					((Collection)va).add(vb);
				}
			}
			else {
				if (vb instanceof Map) {
					throw new IllegalArgumentException("can't merge instance of "+va.getClass()+" to map (key was: "+k+")");
				}
				else if (vb instanceof Collection) {
					throw new IllegalArgumentException("can't merge instance of "+va.getClass()+" to collection (key was: "+k+")");
				}
				
				a.put(k, vb);
			}
		}
		
		return a;
	}

	public static boolean contentEquals(Collection a, Collection b) {
		if (a==b) return true;
		if (a==null || b==null) return false;
		
		if (a.size()!=b.size()) return false;
		
		for (Object x: a) {
			if (!b.contains(x)) return false;
		}
		
		return true;
	}

	public static <T> Iterable<T> toCleanIterable(Iterable<T> list, boolean allowNull, boolean allowDupes) {
		ArrayList<T> ff = new ArrayList<T>();
		
		for (T f : list) {
			if (!allowNull && f==null) continue;
			if (!allowDupes && ff.contains(f)) continue;
			ff.add(f);
		}
		
		return ff;
	}

	public static <T> List<T> asCleanList(T[] list, boolean allowNull, boolean allowDupes) {
		ArrayList<T> ff = new ArrayList<T>(list.length);
		
		for (T f : list) {
			if (!allowNull && f==null) continue;
			if (!allowDupes && ff.contains(f)) continue;
			ff.add(f);
		}
		
		return ff;
	}

	@SuppressWarnings("unchecked")
	public static <T> T[] toCleanArray(T[] list, Class<T> type, boolean allowNull, boolean allowDupes) {
		List<T> ff = asCleanList(list, allowNull, allowDupes);
		return (T[]) ff.toArray((T[])Array.newInstance(type, ff.size()));
	}
}
